/*
** callbacks.c
** 
** Made by (Arne Caspari)
** Login   <arne@arne-laptop>
** 
** Started on  Wed Feb 21 18:18:28 2007 Arne Caspari
** Last update Wed Mar 21 18:30:17 2007 Arne Caspari
*/

#include <glib.h>
#include <gtk/gtk.h>
#include <gconf/gconf-client.h>

#include <unicap.h>
#include <unicapgtk.h>
#include <ucil.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ucview.h"
#include "callbacks.h"


gboolean display_motion_notify_cb( GtkWidget *widget, GdkEventMotion *event, new_frame_data_t *data )
{
   gint x,y;
   GdkModifierType state;
   
   gdk_window_get_pointer( event->window, &x, &y, &state );

   if( x >= data->format.size.width )
   {
      x = data->format.size.width-1;
   }
   
   if( y >= data->format.size.height )
   {
      y = data->format.size.height -1;
   }
   

   data->line_points[2] = x;
   data->line_points[3] = y;
   
   if( !data->line_start_point_set )
   {
      data->line_points[0] = x;
      data->line_points[1] = y;
   }

   return TRUE;
}

gboolean display_button_press_event_cb( GtkWidget *widget, GdkEventButton *event, new_frame_data_t *data )
{
   if( event->button == 1 )
   {
      data->line_start_point_set = TRUE;
      data->line_points[0] = event->x;
      data->line_points[1] = event->y;
   }
   
   return TRUE;
}

void fps_toggled_cb( GtkAction *action, new_frame_data_t *data )
{
   g_mutex_lock( data->mutex );
   if( gtk_toggle_action_get_active( GTK_TOGGLE_ACTION( action ) ) )
   {
      image_effect_data_t *fx_data;
      image_effect_data_t *old_data;
      fx_data = g_malloc( sizeof( image_effect_data_t ) );
      fx_data->func = ifx_print_fps;
      fx_data->name = "print fps";
      old_data = ifx_list_put( &data->image_effects, fx_data );
      g_free( old_data );
   }
   else
   {
      image_effect_data_t *old_data;
      
      if( ( old_data = ifx_list_remove( &data->image_effects, "print fps" ) ) )
      {
	 g_free( old_data );
      }
   }
   g_mutex_unlock( data->mutex );
}

void measure_toggled_cb( GtkAction *action, new_frame_data_t *data )
{
   data->line_points[0] = data->line_points[1] = data->line_points[2] = data->line_points[3] = 0;
   
   g_mutex_lock( data->mutex );
   if( gtk_toggle_action_get_active( GTK_TOGGLE_ACTION( action ) ) )
   {
      image_effect_data_t *fx_data;
      image_effect_data_t *old_data;
      fx_data = g_malloc( sizeof( image_effect_data_t ) );
      fx_data->func = (image_effect_func_t)ifx_draw_measure;
      fx_data->name = "line";
      fx_data->data = data->line_points;
      old_data = ifx_list_put( &data->image_effects, fx_data );
      g_free( old_data );
      data->line_start_point_set = FALSE;
   }
   else
   {
      image_effect_data_t *old_data;
      
      if( ( old_data = ifx_list_remove( &data->image_effects, "line" ) ) )
      {
	 g_free( old_data );
      }
   }
   g_mutex_unlock( data->mutex );   
}

gint color_mag_window_timeout( color_mag_window_data_t *data )
{
   int i,j;
   int w,h;
   ucil_color_t white;
   

   w = data->data_buffer.format.size.width / COLOR_MAG_COLS;
   h = data->data_buffer.format.size.height / COLOR_MAG_ROWS;

   white.colorspace = UCIL_COLORSPACE_RGB24;
   white.rgb24.r = 0xff;
   white.rgb24.g = 0xff;
   white.rgb24.b = 0xff;

   for( i = 0; i < COLOR_MAG_COLS; i++ )
   {
      for( j = 0; j < COLOR_MAG_ROWS; j++ )
      {
	 ucil_color_t color;
	 char buffer[128];
	 
	 color.colorspace = UCIL_COLORSPACE_RGB24;
	 
	 ucil_convert_color( &color, &data->colors[i][j] );
	 ucil_draw_rect( &data->data_buffer, &color, i * w, j * h, i * w + ( w-1 ), j * h + ( h-1 ) );

	 sprintf( buffer, "r:% 3d", color.rgb24.r );
	 ucil_draw_text( &data->data_buffer, &white, data->fobj, buffer, i*w+data->hspacing, j*h+data->vspacing );
	 sprintf( buffer, "g:% 3d", color.rgb24.g );
	 ucil_draw_text( &data->data_buffer, &white, data->fobj, buffer, i*w+data->hspacing, j*h+2*data->vspacing+data->text_h );
	 sprintf( buffer, "b:% 3d", color.rgb24.b );
	 ucil_draw_text( &data->data_buffer, &white, data->fobj, buffer, i*w+data->hspacing, j*h+3*data->vspacing+2*data->text_h );
      }
   }

   gdk_threads_enter();
   unicapgtk_video_display_set_still_image( UNICAPGTK_VIDEO_DISPLAY( data->display ), &data->data_buffer );
   gdk_threads_leave();
   
   return TRUE;
}

void record_toggled_cb( GtkAction *action, new_frame_data_t *data )
{
   if( gtk_toggle_action_get_active( GTK_TOGGLE_ACTION( action ) ) )
   {
      gchar *path;
      int i;
      gchar *filename;
      
      path = gconf_client_get_string( data->client, UCVIEW_GCONF_DIR "/video_file_path", NULL );
      if( !g_file_test( path, ( G_FILE_TEST_IS_DIR ) ) )
      {
	 return;
      }

      for( i = 0; i < 99999; i++ )
      {
	 char num[5];
	 gchar *tmpname;
	 sprintf( num, "%04d", i );
	 tmpname = g_strconcat( "Video_", num, ".", ucil_get_video_file_extension( NULL ), NULL );
	 filename = g_build_path( G_DIR_SEPARATOR_S, path, tmpname, NULL );
	 g_free( tmpname );
	 if( !g_file_test( filename, G_FILE_TEST_EXISTS ) )
	 {
	    break;
	 }
      }
      
      data->video_object = ucil_create_video_file( filename, &data->format, NULL, "fill_frames", 1, NULL );
      if( data->video_object )
      {
	 data->record = TRUE;
	 if( data->last_video_file )
	 {
	    g_free( data->last_video_file );
	 }
	 data->last_video_file = filename;
      }
   }
   else
   {
      data->record = FALSE;
      ucil_close_video_file( data->video_object );
   }      
}

void play_toggled_cb( GtkAction *action, new_frame_data_t *data )
{
   if( gtk_toggle_action_get_active( GTK_TOGGLE_ACTION( action ) ) )
   {
      if( data->last_video_file )
      {
	 if( SUCCESS( ucil_open_video_file( &data->video_file_handle, data->last_video_file ) ) )
	 {
	    if( SUCCESS( unicapgtk_video_display_set_handle( UNICAPGTK_VIDEO_DISPLAY( data->display ), data->video_file_handle ) ) )
	    {
	       unicapgtk_video_display_start( UNICAPGTK_VIDEO_DISPLAY( data->display ) );
	    }
	 }
      }
   }
   else
   {
      unicapgtk_video_display_set_handle( UNICAPGTK_VIDEO_DISPLAY( data->display ), data->device_handle );
      unicapgtk_video_display_start( UNICAPGTK_VIDEO_DISPLAY( data->display ) );
   }    
}

void save_still_image_cb( GtkAction *action, new_frame_data_t *data )
{
   gchar *path;
   int i;
   gchar filename[1024];
   
   path = gconf_client_get_string( data->client, UCVIEW_GCONF_DIR "/video_file_path", NULL );
   if( !g_file_test( path, ( G_FILE_TEST_IS_DIR ) ) )
   {
      return;
   }
   
   for( i = 0; i < 99999; i++ )
   {
      sprintf( filename, "%s%cImage_%05d.jpg", path, G_DIR_SEPARATOR, i );
      if( !g_file_test( filename, G_FILE_TEST_EXISTS ) )
      {
	 break;
      }
   }

   strcpy( data->still_image_path, filename );
   data->still_image = TRUE;
}


void color_mag_toggled_cb( GtkAction *action, new_frame_data_t *data )
{

   data->line_points[0] = data->line_points[1] = data->line_points[2] = data->line_points[3] = 0;

   g_mutex_lock( data->mutex );
   if( gtk_toggle_action_get_active( GTK_TOGGLE_ACTION( action ) ) )
   {
      image_effect_data_t *fx_data;
      image_effect_data_t *old_data;
      color_mag_window_data_t *cw_data;
      GtkWidget *cw_window;
      gint w,h;
      
      cw_data = g_malloc( sizeof( color_mag_window_data_t ) );

      fx_data = g_malloc( sizeof( image_effect_data_t ) );
      fx_data->func = (image_effect_func_t)ifx_update_color_mag;
      fx_data->name = "color mag";
      fx_data->data = cw_data;
      cw_data->cursor_pos = data->line_points;
      cw_data->fobj = ucil_create_font_object( 12, NULL );

      cw_window = ifx_create_color_mag_window( NULL, COLOR_MAG_COLS, COLOR_MAG_ROWS, cw_data );
      cw_data->window = cw_window;
      gtk_widget_show_all( cw_window );
      gtk_widget_get_size_request( cw_data->display, &w, &h );
      g_assert( w > 0 );
      g_assert( h > 0 );
      cw_data->data_buffer.format.fourcc = UCIL_FOURCC( 'R', 'G', 'B', '3' );
      cw_data->data_buffer.format.bpp = 24;
      cw_data->data_buffer.format.size.width = w;
      cw_data->data_buffer.format.size.height = h;
      cw_data->data_buffer.buffer_size = w * h * 3;
      cw_data->data_buffer.data = malloc( cw_data->data_buffer.buffer_size );

      unicapgtk_video_display_set_format( UNICAPGTK_VIDEO_DISPLAY( cw_data->display ), &cw_data->data_buffer.format );
      
      old_data = ifx_list_put( &data->image_effects, fx_data );
      g_free( old_data );
      data->line_start_point_set = FALSE;
      
      cw_data->timeout_tag = g_timeout_add( 250, (GSourceFunc) color_mag_window_timeout, cw_data );
   }
   else
   {
      image_effect_data_t *old_data;
      
      if( ( old_data = ifx_list_remove( &data->image_effects, "color mag" ) ) )
      {
	 color_mag_window_data_t *cw_data = ( color_mag_window_data_t *) old_data->data;
	 g_source_remove( cw_data->timeout_tag );
	 gtk_widget_destroy( cw_data->window );
	 ucil_destroy_font_object( cw_data->fobj );
	 free( cw_data->data_buffer.data );
	 g_free( cw_data );
	 g_free( old_data );
      }
   }
   g_mutex_unlock( data->mutex );   
}

void pause_state_toggled_cb( GtkAction *action, new_frame_data_t *data )
{
   unicapgtk_video_display_set_pause( UNICAPGTK_VIDEO_DISPLAY( data->display ), 
				      gtk_toggle_action_get_active( GTK_TOGGLE_ACTION( action ) ) );
}

void display_property_dialog_cb( GtkAction *action, new_frame_data_t *data )
{
   gtk_window_present( GTK_WINDOW( data->property_dialog ) );
}

void display_settings_dialog_cb( GtkAction *action, new_frame_data_t *data )
{
   gtk_window_present( GTK_WINDOW( data->settings_dialog ) );
}
