/*
** gui.c
** 
*/

#include <gtk/gtk.h>

#include "ucview.h"
#include "gui.h"
#include "callbacks.h"

static void activate_action( GtkAction *action, new_frame_data_t *data );
static void activate_toggle_action( GtkAction *action, new_frame_data_t *data );

static const gchar *ui_info = 
"<ui>"
" <menubar name='MenuBar'>"
"  <menu action='FileMenu'>"
"   <menuitem action='Quit'/>"
"  </menu>"
"  <menu action='EditMenu'>"
"   <menuitem action='Copy'/>"
"   <separator/>"
"   <menuitem action='DeviceSettings'/>"
"   <menuitem action='Preferences'/>"
"  </menu>"
"  <menu action='ViewMenu'>"
"   <menuitem action='ViewToolbar'/>"
"   <separator/>"
"   <menuitem action='Pause'/>"
"  </menu>"
"  <menu action='ImageMenu'>"
"   <menuitem action='ShowFPS'/>"
"   <menuitem action='Measure'/>"
/* "   <menuitem action='ColorMag'/>" */
"  </menu>"
" </menubar>"
" <toolbar name='ToolBar'>"
"   <toolitem action='Pause'/>"
"   <toolitem action='SaveImage'/>"
"   <toolitem action='RecordVideo'/>"
"   <toolitem action='PlayVideo'/>"
"   <separator/>"
"   <toolitem action='DeviceSettings'/>"
"   <toolitem action='Preferences'/>"
" </toolbar>"
"</ui>";

static GtkActionEntry entries[] =
{
   { "FileMenu", NULL, "_File" }, 
   { "EditMenu", NULL, "_Edit" },
   { "ViewMenu", NULL, "_View" },
   { "ImageMenu", NULL, "_Image" },

   { "Quit", GTK_STOCK_QUIT, "_Quit", NULL, "Quit", G_CALLBACK( gtk_main_quit ) }, 
   { "Copy", GTK_STOCK_COPY, "_Copy", "<control>C", "Copy", G_CALLBACK( activate_action ) },
   { "DeviceSettings", NULL, "_Device Settings", NULL, "Device Settings", G_CALLBACK( display_property_dialog_cb ) }, 
   { "Preferences", GTK_STOCK_PREFERENCES, "_Preferences", NULL, "Preferences", G_CALLBACK( display_settings_dialog_cb ) }, 

   { "SaveImage", NULL, "_SaveImage", "<control>S", "Save Still Image", G_CALLBACK( save_still_image_cb ) },
   
};
static const guint n_entries = G_N_ELEMENTS( entries );

static GtkToggleActionEntry toggle_entries[] = 
{
   { "ViewToolbar", NULL, "_Toolbar", NULL, "Toolbar", G_CALLBACK( activate_toggle_action ), TRUE }, 

   { "Pause", NULL, "_Pause", "<control>P", "Pause", G_CALLBACK( pause_state_toggled_cb ) },
   { "RecordVideo", NULL, "_RecordVideo", "<control>R", "Record Video", G_CALLBACK( record_toggled_cb ), FALSE },
   { "PlayVideo", NULL, "_PlayVideo", "<control>P", "Play Video", G_CALLBACK( play_toggled_cb ), FALSE },

   { "ShowFPS", NULL, "Show _FPS", NULL, "Show FPS", G_CALLBACK( fps_toggled_cb ) },
   { "Measure", NULL, "_Measure", NULL, "Measure", G_CALLBACK( measure_toggled_cb ) },
   { "ColorMag", NULL, "_Color Magnifier", NULL, "Color Magnifier", G_CALLBACK( color_mag_toggled_cb ) },
   
};
static const guint n_toggle_entries = G_N_ELEMENTS( toggle_entries );


static void activate_action( GtkAction *action, new_frame_data_t *data )
{
   g_message( "Action '%s' activated\n", gtk_action_get_name( action ) );
}

static void activate_toggle_action( GtkAction *action, new_frame_data_t *data )
{
   g_message( "ToggleAction '%s' %s\n", gtk_action_get_name( action ), gtk_toggle_action_get_active( GTK_TOGGLE_ACTION( action ) ) ? "activated" : "deactivated" );
}



GtkUIManager *gui_create_manager( new_frame_data_t *data )
{
   GtkUIManager *ui;
   GtkActionGroup *actions;
   GError *error = NULL;
   
   actions = gtk_action_group_new( "Actions" );
   gtk_action_group_add_actions( actions, entries, n_entries, data );
   gtk_action_group_add_toggle_actions( actions, toggle_entries, n_toggle_entries, data );
   
   ui = gtk_ui_manager_new();
   gtk_ui_manager_insert_action_group( ui, actions, 0 );
   g_object_unref( actions );
   
   if( !gtk_ui_manager_add_ui_from_string( ui, ui_info, -1, &error ) )
   {
      g_error( "Building menus failed: %s\n", error->message );
      g_error_free( error );
   }

   return ui;
}

